@@ -18,6 +18,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
 
 #include "mediastreamer2/mssndcard.h"
 #include "mediastreamer2/msfilter.h"
@@ -30,7 +33,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+
+#ifdef HAVE_ALLOCA_H /*FreeBSD does not have alloca.h*/
 #include <alloca.h>
+#endif
 
 MSFilter *ms_oss_read_new(MSSndCard *card);
 MSFilter *ms_oss_write_new(MSSndCard *card);
@@ -264,6 +270,8 @@ MSSndCardDesc oss_card_desc={
 	.set_level=oss_set_level,
 	.get_level=oss_get_level,
 	.set_capture=oss_set_source,
+	.set_control=NULL,
+	.get_control=NULL,
 	.create_reader=ms_oss_read_new,
 	.create_writer=ms_oss_write_new,
 	.uninit=oss_uninit,
@@ -296,8 +304,6 @@ static void oss_detect(MSSndCardManager *m){
 	if (access(DSP_NAME,F_OK)==0){
 		MSSndCard *card=oss_card_new(DSP_NAME,MIXER_NAME);
 		ms_snd_card_manager_add_card(m,card);
-		card=oss_card_new(DSP_NAME,MIXER_NAME);
-		ms_snd_card_manager_add_card(m,card);
 	}
 	for(i=0;i<10;i++){
 		snprintf(pcmdev,sizeof(pcmdev),"%s%i",DSP_NAME,i);
@@ -319,8 +325,13 @@ static void * oss_thread(void *p){
 	mblk_t *rm=NULL;
 	d->pcmfd=oss_open(d->pcmdev,d->bits,d->stereo,d->rate,&bsize);
 	if (d->pcmfd>=0){
-		rtmpbuff=(uint8_t*)alloca(bsize);
-		wtmpbuff=(uint8_t*)alloca(bsize);
+		rtmpbuff=(uint8_t*)malloc(bsize);
+		wtmpbuff=(uint8_t*)malloc(bsize);
+		if(rtmpbuff == NULL || wtmpbuff == NULL) {
+			free(rtmpbuff);
+			free(wtmpbuff);
+			return NULL;
+		}
 	}
 	while(d->read_started || d->write_started){
 		if (d->pcmfd>=0){
@@ -378,10 +389,12 @@ static void * oss_thread(void *p){
 						/* drop the fragment if the buffer starts to fill up */
 						/* we got too much data: I prefer to empty the incoming buffer */
 						while (ms_bufferizer_get_avail(d->bufferizer)>bsize*4){
+							ms_mutex_lock(&d->mutex);
 							err=ms_bufferizer_read(d->bufferizer,wtmpbuff,bsize);
 							err=ms_bufferizer_read(d->bufferizer,wtmpbuff,bsize);
 							err=ms_bufferizer_read(d->bufferizer,wtmpbuff,bsize);
 							err=ms_bufferizer_read(d->bufferizer,wtmpbuff,bsize);
+							ms_mutex_unlock(&d->mutex);
 							c=c+err*4;
 							ms_warning("drop fragment when buffer gets too much data (%i - discarded:%i)", info.fragstotal - info.fragments, c);
 							if (err==0)
@@ -389,7 +402,9 @@ static void * oss_thread(void *p){
 						}
 
 					}else {
+						ms_mutex_lock(&d->mutex);
 						err=ms_bufferizer_read(d->bufferizer,wtmpbuff,bsize);
+						ms_mutex_unlock(&d->mutex);
 						err=write(d->pcmfd,wtmpbuff,bsize);
 						if (err<0){
 							ms_warning("Fail to write %i bytes from soundcard: %s",
@@ -410,20 +425,25 @@ static void * oss_thread(void *p){
 		close(d->pcmfd);
 		d->pcmfd=-1;
 	}
+	free(rtmpbuff);
+	free(wtmpbuff);
 	if (rm!=NULL) freemsg(rm);
 	/*reset to default parameters */
-	d->bits=16;
-	d->rate=8000;
-	d->stereo=FALSE;
+	//d->bits=16;
+	//d->rate=8000;
+	//d->stereo=FALSE;
 	return NULL;
 }
 
 static void oss_start_r(MSSndCard *card){
 	OssData *d=(OssData*)card->data;
+	ms_mutex_lock(&d->mutex);
 	if (d->read_started==FALSE && d->write_started==FALSE){
 		d->read_started=TRUE;
 		ms_thread_create(&d->thread,NULL,oss_thread,card);
 	}else d->read_started=TRUE;
+	flushq(&d->rq,0);
+	ms_mutex_unlock(&d->mutex);
 }
 
 static void oss_stop_r(MSSndCard *card){
@@ -434,14 +454,22 @@ static void oss_stop_r(MSSndCard *card){
 	}
 }
 
+static void _flush_buffer(MSBufferizer *obj){
+	flushq(&obj->q,0);
+	obj->size=0;
+}
+
 static void oss_start_w(MSSndCard *card){
 	OssData *d=(OssData*)card->data;
+	ms_mutex_lock(&d->mutex);
 	if (d->read_started==FALSE && d->write_started==FALSE){
 		d->write_started=TRUE;
 		ms_thread_create(&d->thread,NULL,oss_thread,card);
 	}else{
 		d->write_started=TRUE;
 	}
+	_flush_buffer(d->bufferizer);
+	ms_mutex_unlock(&d->mutex);
 }
 
 static void oss_stop_w(MSSndCard *card){
@@ -505,6 +533,13 @@ static void oss_write_process(MSFilter *f){
 	}
 }
 
+static int get_rate(MSFilter *f, void *arg){
+	MSSndCard *card=(MSSndCard*)f->data;
+	OssData *d=(OssData*)card->data;
+	*((int*)arg)=d->rate;
+	return 0;
+}
+
 static int set_rate(MSFilter *f, void *arg){
 	MSSndCard *card=(MSSndCard*)f->data;
 	OssData *d=(OssData*)card->data;
@@ -520,6 +555,7 @@ static int set_nchannels(MSFilter *f, void *arg){
 }
 
 static MSFilterMethod oss_methods[]={
+	{	MS_FILTER_GET_SAMPLE_RATE	, get_rate	},
 	{	MS_FILTER_SET_SAMPLE_RATE	, set_rate	},
 	{	MS_FILTER_SET_NCHANNELS		, set_nchannels	},
 	{	0				, NULL		}
@@ -528,7 +564,7 @@ static MSFilterMethod oss_methods[]={
 MSFilterDesc oss_read_desc={
 	.id=MS_OSS_READ_ID,
 	.name="MSOssRead",
-	.text="Sound capture filter for OSS drivers",
+	.text=N_("Sound capture filter for OSS drivers"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=0,
 	.noutputs=1,
@@ -542,7 +578,7 @@ MSFilterDesc oss_read_desc={
 MSFilterDesc oss_write_desc={
 	.id=MS_OSS_WRITE_ID,
 	.name="MSOssWrite",
-	.text="Sound playback filter for OSS drivers",
+	.text=N_("Sound playback filter for OSS drivers"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=1,
 	.noutputs=0,
